@@ -1,10 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "mindstream" "2.0"
+(define-package "mindstream" "20250821.2237"
   "Start writing, stay focused, don't worry."
-  '((emacs "28.1"))
+  '((emacs "26.1")
+    (magit "3.3.0"))
   :url "https://github.com/countvajhula/mindstream"
-  :commit "6b51036a069379d18538a7cca5274c21666a5979"
-  :revdesc "v2.0-0-g6b51036a0693"
+  :commit "f76934cdda1c5aa3cf6e64c7ffab350ee97bc22d"
+  :revdesc "f76934cdda1c"
   :keywords '("convenience" "files" "languages" "outlines" "tools" "vc" "wp")
   :authors '(("Siddhartha Kasivajhula" . "sid@countvajhula.com"))
   :maintainers '(("Siddhartha Kasivajhula" . "sid@countvajhula.com")))
