@@ -2,9 +2,9 @@
 
 ;; Author: Siddhartha Kasivajhula <sid@countvajhula.com>
 ;; URL: https://github.com/countvajhula/mindstream
-;; Package-Version: 2.0
-;; Package-Revision: v2.0-0-g6b51036a0693
-;; Package-Requires: ((emacs "28.1"))
+;; Package-Version: 20250821.2237
+;; Package-Revision: f76934cdda1c
+;; Package-Requires: ((emacs "26.1") (magit "3.3.0"))
 ;; Keywords: convenience, files, languages, outlines, tools, vc, wp
 
 ;; This work is "part of the world." You are free to do whatever you
@@ -39,8 +39,10 @@
 
 ;;; Code:
 
+(require 'magit-git)
+(require 'dired)
+
 (require 'mindstream-custom)
-(require 'mindstream-stream)
 (require 'mindstream-session)
 (require 'mindstream-backend)
 (require 'mindstream-util)
@@ -65,7 +67,8 @@ can be retrieved and canceled when you leave live mode.")
     (define-key mindstream-map (kbd "C-c , n") #'mindstream-new)
     (define-key mindstream-map (kbd "C-c , b") #'mindstream-enter-anonymous-session)
     (define-key mindstream-map (kbd "C-c , t") #'mindstream-enter-session-for-template)
-    (define-key mindstream-map (kbd "C-c , m") #'mindstream-start-stream)
+    (define-key mindstream-map (kbd "C-c , m") #'mindstream-begin-session)
+    (define-key mindstream-map (kbd "C-c , q") #'mindstream-end-session)
     (define-key mindstream-map (kbd "C-c , s") #'mindstream-save-session)
     (define-key mindstream-map (kbd "C-c , C-s") #'mindstream-save-session)
     (define-key mindstream-map (kbd "C-c , r") #'mindstream-load-session)
@@ -101,6 +104,17 @@ For example:
     (completing-read "Which template? " templates nil t nil
                      'mindstream-template-history)))
 
+(defun mindstream--new (template)
+  "Start a new anonymous session using a specific TEMPLATE.
+
+This also begins a new session."
+  ;; start a new session (sessions always start anonymous)
+  (let ((buf (mindstream-start-anonymous-session template)))
+    ;; (ab initio) iterate
+    (with-current-buffer buf
+      (mindstream--iterate))
+    buf))
+
 (defun mindstream-new (&optional template)
   "Start a new anonymous session.
 
@@ -117,11 +131,10 @@ name, in a dedicated Git version-controlled folder at
   (interactive)
   (let ((template (or template
                       (mindstream--completing-read-template))))
-    ;; start a new session (sessions always start anonymous)
-    (switch-to-buffer (mindstream-start-anonymous-session template))))
+    (switch-to-buffer (mindstream--new template))))
 
 (defun mindstream-start-anonymous-session (&optional template)
-  "Start a new anonymous session using a specific TEMPLATE.
+  "Start a new anonymous session.
 
 This creates a new directory and Git repository for the new session
 after copying over the contents of TEMPLATE if one is specified.
@@ -140,16 +153,14 @@ New sessions always start anonymous."
        (expand-file-name filename
                          path))
       (mindstream--initialize-buffer)
-      ;; (ab initio) iterate
-      (mindstream--iterate)
-      (mindstream-start-stream)
+      (mindstream-begin-session)
       (current-buffer))))
 
 (defun mindstream-initialize ()
   "Do any setup that's necessary for Mindstream.
 
-This subscribes to any hooks that should implicitly cause the session to
-iterate.  By default, this is just `after-save-hook', so that the
+This advises any functions that should implicitly cause the session to
+iterate.  By default, this is just `basic-save-buffer', so that the
 session is iterated every time the buffer is saved.  This is the
 recommended usage, intended to capture \"natural\" points at which the
 session is meaningful.
@@ -158,35 +169,27 @@ While it doesn't make sense to iterate the session if the buffer
 has *not* been saved (there would be no changes to record a fresh
 version for!), it's possible that you might want to iterate the
 session at a coarser granularity than every save. In that case, you
-can customize `mindstream-triggers' and add the hook(s) that
-should trigger session iteration (and remove `after-save-hook')."
+can customize `mindstream-triggers' and add the function(s) that
+should trigger session iteration (and remove `basic-save-buffer')."
   (mindstream--ensure-paths)
   (mindstream--ensure-templates-exist)
   (unless mindstream-persist
     ;; archive all sessions on startup
     (mindstream-archive-all))
-  (dolist (trigger mindstream-triggers)
-    (if (mindstream--is-hook-p trigger)
-        (add-hook trigger #'mindstream-implicitly-iterate)
-      ;; Backward compatibility for legacy advice-based configs
-      (display-warning 'mindstream
-                       (format "Advising `%s' is deprecated. Please update `mindstream-triggers' to use hooks (e.g., `after-save-hook')." trigger)
-                       :warning)
-      (advice-add trigger :after #'mindstream-implicitly-iterate))))
+  (dolist (fn mindstream-triggers)
+    (advice-add fn :after #'mindstream-implicitly-iterate-advice)))
 
 (defun mindstream-disable ()
   "Cleanup actions on exiting `mindstream-mode'.
 
-This unsubscribes from any hooks (e.g. `after-save-hook') for session
-iteration.  It also removes advice (deprecated), if present."
-  (dolist (trigger mindstream-triggers)
-    (if (mindstream--is-hook-p trigger)
-        (remove-hook trigger #'mindstream-implicitly-iterate)
-      (advice-remove trigger #'mindstream-implicitly-iterate))))
+This removes any advice (e.g. on `basic-save-buffer') that was added
+for session iteration."
+  (dolist (fn mindstream-triggers)
+    (advice-remove fn #'mindstream-implicitly-iterate-advice)))
 
 (defun mindstream--call-live-action ()
   "Call configured live action for major mode."
-  (when (and (mindstream-stream-p)
+  (when (and (mindstream-session-p)
              (boundp 'mindstream-live-timer)
              mindstream-live-timer)
     (let ((action (plist-get mindstream-live-action
@@ -210,7 +213,7 @@ iteration.  It also removes advice (deprecated), if present."
 
 (defun mindstream--reset-live-timer (_beg _end _len)
   "Reset the live timer."
-  (when (mindstream-stream-p)
+  (when (mindstream-session-p)
     (mindstream--cancel-live-timer)
     (mindstream--start-live-timer)))
 
@@ -236,27 +239,14 @@ before invoking it is customized via `mindstream-live-delay'."
                #'mindstream--reset-live-timer
                t))
 
-(defun mindstream--is-hook-p (sym)
-  "Check if SYM follows the Emacs convention for hook variables."
-  (string-match-p "-\\(hook\\|functions\\)\\'" (symbol-name sym)))
+(defun mindstream-implicitly-iterate-advice (&rest _)
+  "Implicitly iterate the session upon execution of some command.
 
-(defun mindstream-implicitly-iterate (&rest _)
-  "Implicitly iterate the session upon execution of a trigger.
-
-Note that the versioning backend, Git, will *not* ordinarily record a
-fresh commit if there are no differences from the previous version, so
-there is typically no risk of empty commits from calling this function
-redundantly.
-
-This accepts `&rest _` so it can be used interchangeably as a standard
-hook (0 args), an abnormal hook, or an advice function (though this
-last is deprecated)."
-  (when (mindstream-stream-p)
+This only iterates the session if there have been changes since
+the last persistent state.  Otherwise, it takes no action."
+  (when (mindstream-session-p)
     (mindstream--iterate)))
 
-(define-obsolete-function-alias 'mindstream-implicitly-iterate-advice
-  'mindstream-implicitly-iterate "2.0")
-
 (defun mindstream-save-session (dest-dir)
   "Save the current session to a permanent location.
 
@@ -294,9 +284,17 @@ to select the existing destination path."
                       ;; not named - retain anonymous session name
                       (mindstream--build-path dest-dir
                                               (mindstream--session-name))
-                    dest-dir)))
+                    dest-dir))
+        (file-to-open (file-name-nondirectory
+                       (buffer-file-name))))
     ;; TODO: verify behavior with existing vs non-existent containing folder
-    (mindstream--move-dir source-dir dest-dir)))
+    (mindstream--move-dir source-dir dest-dir)
+    ;; TODO: this is a no-op, and it currently leaves the
+    ;; session in `mindsteam-active-sessions'. But that's OK
+    ;; for now as it doesn't affect anything, and this "state"
+    ;; will be removed eventually anyway in the design refactor
+    ;; (mindstream--end-anonymous-session)
+    (mindstream-load-session dest-dir file-to-open)))
 
 (defun mindstream-load-session (dir &optional file)
   "Load a previously saved session.
@@ -310,12 +308,7 @@ the file to be opened."
   (let ((file (or file
                   (mindstream--starting-file-for-session dir))))
     (find-file (expand-file-name file dir))
-    (if (mindstream-stream-p)
-        ;; if already in a stream (i.e., currently on a mindstream
-        ;; branch), we don't need to do anything, but we do still call
-        ;; the "helper" here to add the session to completion history
-        (mindstream--start-stream-helper)
-      (mindstream--start-stream))))
+    (mindstream-begin-session)))
 
 (defun mindstream--completing-read-session ()
   "Return session-file via completion for template."
@@ -387,7 +380,7 @@ worrying about how that happens. It is too connoted to be useful in
 features implementing the session iteration model."
   (or (mindstream--visit-anonymous-session template)
       (mindstream-open template)
-      (mindstream-start-anonymous-session template)))
+      (mindstream--new template)))
 
 (defun mindstream-enter-session-for-template (template)
   "Enter an anonymous session buffer for TEMPLATE.
@@ -411,18 +404,18 @@ present, otherwise, it creates a new one and enters it."
     major-mode)))
 
 (defun mindstream--list-anonymous-sessions ()
-  "List anonymous session paths, ordered by recency of buffer use."
+  "List anonymous session paths."
   (let ((sessions nil))
     (mindstream--for-all-buffers
      (lambda ()
        (when (mindstream-anonymous-session-p)
          (push (mindstream--session-dir (current-buffer))
                sessions))))
-    ;; `buffer-list` is already sorted most-recently-used first.
-    ;; `push` reverses this, so `nreverse` restores the true MRU order.
-    ;; `seq-uniq` keeps the first occurrence, maintaining recency priority.
-    (let ((sessions (seq-uniq (nreverse sessions))))
-      (when (mindstream-stream-p)
+    (let ((sessions (sort (seq-uniq sessions)
+                          #'mindstream--files-sort-by-modified-time)))
+      (when (mindstream-session-p)
+        ;; ensure the session in the current buffer
+        ;; is at the top of the completion menu
         (let ((this-session (mindstream--session-dir)))
           (setq sessions
                 (cons this-session
@@ -450,24 +443,18 @@ present, otherwise, it creates a new one and enters it."
     (when (mindstream--directory-empty-p anon-date-dir)
       (delete-directory anon-date-dir))))
 
+;; TODO: archive should be ordered by recency, so that the current session is highlighted.
 (defun mindstream-archive (session)
   "Move the selected SESSION to `mindstream-archive-path'.
 
 The session is expected to be anonymous - it does not make sense to
 archive saved sessions."
-  (interactive
-   (let* ((sessions (mindstream--list-anonymous-sessions))
-          ;; Grab the first item (current or most recent) to use as default
-          (default (car sessions)))
-     (list (completing-read (if default
-                                (format "Which session? (default %s): " default)
-                              "Which session? ")
-                            sessions
-                            nil
-                            t
-                            nil
-                            'mindstream-session-history
-                            default))))
+  (interactive (list (completing-read "Which session? "
+                                      (mindstream--list-anonymous-sessions)
+                                      nil
+                                      t
+                                      nil
+                                      'mindstream-session-history)))
   (let ((template (mindstream--template-used session)))
     (mindstream--archive session
                          template)))
